@@ -657,6 +657,7 @@ module blinkled
   assign myaxi_arprot = 0;
   assign myaxi_arqos = 0;
   assign myaxi_aruser = 0;
+  reg [32-1:0] outstanding_wreq_count_0;
   reg _myaxi_read_start;
   reg [8-1:0] _myaxi_read_op_sel;
   reg [32-1:0] _myaxi_read_local_addr;
@@ -686,7 +687,7 @@ module blinkled
   reg signed [32-1:0] _th_matmul_a_addr_9;
   reg signed [32-1:0] _th_matmul_c_addr_10;
   reg signed [32-1:0] _th_matmul_i_11;
-  reg axim_flag_0;
+  reg axim_flag_1;
   reg [32-1:0] _d1_th_matmul;
   reg _th_matmul_cond_6_0_1;
   reg _myaxi_ram_a_0_read_start;
@@ -700,30 +701,30 @@ module blinkled
   reg [32-1:0] _myaxi_read_cur_global_addr;
   reg [33-1:0] _myaxi_read_cur_size;
   reg [33-1:0] _myaxi_read_rest_size;
-  reg [32-1:0] _wdata_1;
-  reg _wvalid_2;
-  reg [33-1:0] _tmp_3;
-  reg _tmp_4;
+  reg [32-1:0] _wdata_2;
+  reg _wvalid_3;
+  reg [33-1:0] _tmp_4;
+  reg _tmp_5;
   wire [32-1:0] _dataflow__variable_odata_0;
   wire _dataflow__variable_ovalid_0;
   wire _dataflow__variable_oready_0;
-  assign _dataflow__variable_oready_0 = (_tmp_3 > 0) && !_tmp_4;
-  reg [10-1:0] _tmp_5;
-  reg [32-1:0] _tmp_6;
-  reg _tmp_7;
-  assign ram_a_0_wdata = (_tmp_7)? _tmp_6 : 'hx;
-  assign ram_a_0_wenable = (_tmp_7)? 1'd1 : 0;
+  assign _dataflow__variable_oready_0 = (_tmp_4 > 0) && !_tmp_5;
+  reg [10-1:0] _tmp_6;
+  reg [32-1:0] _tmp_7;
+  reg _tmp_8;
+  assign ram_a_0_wdata = (_tmp_8)? _tmp_7 : 'hx;
+  assign ram_a_0_wenable = (_tmp_8)? 1'd1 : 0;
   reg _ram_a_cond_0_1;
-  reg [9-1:0] _tmp_8;
+  reg [9-1:0] counter_9;
   reg _myaxi_cond_0_1;
   assign myaxi_rready = _myaxi_read_fsm == 3;
   reg [32-1:0] _d1__myaxi_read_fsm;
   reg __myaxi_read_fsm_cond_3_0_1;
-  reg axim_flag_9;
+  reg axim_flag_10;
   reg __myaxi_read_fsm_cond_4_1_1;
   reg signed [32-1:0] _th_matmul_b_addr_12;
   reg signed [32-1:0] _th_matmul_j_13;
-  reg axim_flag_10;
+  reg axim_flag_11;
   reg _th_matmul_cond_13_1_1;
   reg _myaxi_ram_b_0_read_start;
   reg [8-1:0] _myaxi_ram_b_0_read_op_sel;
@@ -731,44 +732,44 @@ module blinkled
   reg [32-1:0] _myaxi_ram_b_0_read_global_addr;
   reg [33-1:0] _myaxi_ram_b_0_read_size;
   reg [32-1:0] _myaxi_ram_b_0_read_local_stride;
-  reg [32-1:0] _wdata_11;
-  reg _wvalid_12;
-  reg [33-1:0] _tmp_13;
-  reg _tmp_14;
+  reg [32-1:0] _wdata_12;
+  reg _wvalid_13;
+  reg [33-1:0] _tmp_14;
+  reg _tmp_15;
   wire [32-1:0] _dataflow__variable_odata_1;
   wire _dataflow__variable_ovalid_1;
   wire _dataflow__variable_oready_1;
-  assign _dataflow__variable_oready_1 = (_tmp_13 > 0) && !_tmp_14;
-  reg [10-1:0] _tmp_15;
-  reg [32-1:0] _tmp_16;
-  reg _tmp_17;
-  assign ram_b_0_wdata = (_tmp_17)? _tmp_16 : 'hx;
-  assign ram_b_0_wenable = (_tmp_17)? 1'd1 : 0;
+  assign _dataflow__variable_oready_1 = (_tmp_14 > 0) && !_tmp_15;
+  reg [10-1:0] _tmp_16;
+  reg [32-1:0] _tmp_17;
+  reg _tmp_18;
+  assign ram_b_0_wdata = (_tmp_18)? _tmp_17 : 'hx;
+  assign ram_b_0_wenable = (_tmp_18)? 1'd1 : 0;
   reg _ram_b_cond_0_1;
   reg __myaxi_read_fsm_cond_3_2_1;
   reg signed [32-1:0] _th_matmul_sum_14;
   reg signed [32-1:0] _th_matmul_k_15;
   assign ram_a_0_addr = (th_matmul == 20)? _th_matmul_k_15 : 
-                        (_tmp_7)? _tmp_5 : 'hx;
+                        (_tmp_8)? _tmp_6 : 'hx;
   assign ram_a_0_enable = (th_matmul == 20)? 1'd1 : 
-                          (_tmp_7)? 1'd1 : 0;
-  localparam _tmp_18 = 1;
-  wire [_tmp_18-1:0] _tmp_19;
-  assign _tmp_19 = th_matmul == 20;
-  reg [_tmp_18-1:0] __tmp_19_1;
-  reg signed [32-1:0] _tmp_20;
+                          (_tmp_8)? 1'd1 : 0;
+  localparam _tmp_19 = 1;
+  wire [_tmp_19-1:0] _tmp_20;
+  assign _tmp_20 = th_matmul == 20;
+  reg [_tmp_19-1:0] __tmp_20_1;
+  reg signed [32-1:0] _tmp_21;
   reg signed [32-1:0] _th_matmul_x_16;
   assign ram_b_0_addr = (th_matmul == 22)? _th_matmul_k_15 : 
-                        (_tmp_17)? _tmp_15 : 'hx;
+                        (_tmp_18)? _tmp_16 : 'hx;
   assign ram_b_0_enable = (th_matmul == 22)? 1'd1 : 
-                          (_tmp_17)? 1'd1 : 0;
-  localparam _tmp_21 = 1;
-  wire [_tmp_21-1:0] _tmp_22;
-  assign _tmp_22 = th_matmul == 22;
-  reg [_tmp_21-1:0] __tmp_22_1;
-  reg signed [32-1:0] _tmp_23;
+                          (_tmp_18)? 1'd1 : 0;
+  localparam _tmp_22 = 1;
+  wire [_tmp_22-1:0] _tmp_23;
+  assign _tmp_23 = th_matmul == 22;
+  reg [_tmp_22-1:0] __tmp_23_1;
+  reg signed [32-1:0] _tmp_24;
   reg signed [32-1:0] _th_matmul_y_17;
-  reg axim_flag_24;
+  reg axim_flag_25;
   reg _th_matmul_cond_29_2_1;
   reg _myaxi_ram_c_0_write_start;
   reg [8-1:0] _myaxi_ram_c_0_write_op_sel;
@@ -781,29 +782,29 @@ module blinkled
   reg [32-1:0] _myaxi_write_cur_global_addr;
   reg [33-1:0] _myaxi_write_cur_size;
   reg [33-1:0] _myaxi_write_rest_size;
-  reg _tmp_25;
   reg _tmp_26;
-  wire _tmp_27;
+  reg _tmp_27;
   wire _tmp_28;
-  assign _tmp_28 = 1;
-  wire signed [32-1:0] _tmp_29;
-  assign _tmp_29 = ram_c_0_rdata;
-  reg _tmp_30;
+  wire _tmp_29;
+  assign _tmp_29 = 1;
+  wire signed [32-1:0] _tmp_30;
+  assign _tmp_30 = ram_c_0_rdata;
   reg _tmp_31;
   reg _tmp_32;
   reg _tmp_33;
-  reg [33-1:0] _tmp_34;
-  reg [10-1:0] _tmp_35;
-  reg [9-1:0] _tmp_36;
+  reg _tmp_34;
+  reg [33-1:0] _tmp_35;
+  reg [10-1:0] _tmp_36;
+  reg [9-1:0] counter_37;
   reg _myaxi_cond_1_1;
-  reg _tmp_37;
+  reg last_38;
   wire [32-1:0] _dataflow__variable_odata_2;
   wire _dataflow__variable_ovalid_2;
   wire _dataflow__variable_oready_2;
-  assign _dataflow__variable_oready_2 = (_myaxi_write_fsm == 3) && (_myaxi_write_op_sel == 1) && ((_tmp_36 > 0) && (myaxi_wready || !myaxi_wvalid));
+  assign _dataflow__variable_oready_2 = (_myaxi_write_fsm == 3) && (_myaxi_write_op_sel == 1) && ((counter_37 > 0) && (myaxi_wready || !myaxi_wvalid));
   reg _myaxi_cond_2_1;
-  assign _myaxi_write_data_done = (_tmp_37 && myaxi_wvalid && myaxi_wready)? 1 : 0;
-  reg axim_flag_38;
+  assign _myaxi_write_data_done = (last_38 && myaxi_wvalid && myaxi_wready)? 1 : 0;
+  reg axim_flag_39;
   reg [32-1:0] _d1__myaxi_write_fsm;
   reg __myaxi_write_fsm_cond_4_0_1;
   reg signed [32-1:0] _th_matmul_end_time_18;
@@ -815,7 +816,7 @@ module blinkled
   reg signed [32-1:0] _th_matmul_all_ok_24;
   reg signed [32-1:0] _th_matmul_c_addr_25;
   reg signed [32-1:0] _th_matmul_i_26;
-  reg axim_flag_39;
+  reg axim_flag_40;
   reg _th_matmul_cond_44_3_1;
   reg _myaxi_ram_c_0_read_start;
   reg [8-1:0] _myaxi_ram_c_0_read_op_sel;
@@ -823,37 +824,37 @@ module blinkled
   reg [32-1:0] _myaxi_ram_c_0_read_global_addr;
   reg [33-1:0] _myaxi_ram_c_0_read_size;
   reg [32-1:0] _myaxi_ram_c_0_read_local_stride;
-  reg [32-1:0] _wdata_40;
-  reg _wvalid_41;
-  reg [33-1:0] _tmp_42;
-  reg _tmp_43;
+  reg [32-1:0] _wdata_41;
+  reg _wvalid_42;
+  reg [33-1:0] _tmp_43;
+  reg _tmp_44;
   wire [32-1:0] _dataflow__variable_odata_4;
   wire _dataflow__variable_ovalid_4;
   wire _dataflow__variable_oready_4;
-  assign _dataflow__variable_oready_4 = (_tmp_42 > 0) && !_tmp_43;
-  reg [10-1:0] _tmp_44;
-  reg [32-1:0] _tmp_45;
-  reg _tmp_46;
-  assign ram_c_0_wdata = (_tmp_46)? _tmp_45 : 
+  assign _dataflow__variable_oready_4 = (_tmp_43 > 0) && !_tmp_44;
+  reg [10-1:0] _tmp_45;
+  reg [32-1:0] _tmp_46;
+  reg _tmp_47;
+  assign ram_c_0_wdata = (_tmp_47)? _tmp_46 : 
                          (th_matmul == 26)? _th_matmul_sum_14 : 'hx;
-  assign ram_c_0_wenable = (_tmp_46)? 1'd1 : 
+  assign ram_c_0_wenable = (_tmp_47)? 1'd1 : 
                            (th_matmul == 26)? 1'd1 : 0;
   reg _ram_c_cond_0_1;
   reg __myaxi_read_fsm_cond_3_3_1;
   reg signed [32-1:0] _th_matmul_j_27;
   assign ram_c_0_addr = (th_matmul == 50)? _th_matmul_j_27 : 
-                        (_tmp_46)? _tmp_44 : 
-                        (_tmp_30)? _tmp_35 : 
+                        (_tmp_47)? _tmp_45 : 
+                        (_tmp_31)? _tmp_36 : 
                         (th_matmul == 26)? _th_matmul_j_13 : 'hx;
   assign ram_c_0_enable = (th_matmul == 50)? 1'd1 : 
-                          (_tmp_46)? 1'd1 : 
-                          ((_tmp_27 || !_tmp_25) && (_tmp_28 || !_tmp_26) && _tmp_30)? 1'd1 : 
+                          (_tmp_47)? 1'd1 : 
+                          ((_tmp_28 || !_tmp_26) && (_tmp_29 || !_tmp_27) && _tmp_31)? 1'd1 : 
                           (th_matmul == 26)? 1'd1 : 0;
-  localparam _tmp_47 = 1;
-  wire [_tmp_47-1:0] _tmp_48;
-  assign _tmp_48 = th_matmul == 50;
-  reg [_tmp_47-1:0] __tmp_48_1;
-  reg signed [32-1:0] _tmp_49;
+  localparam _tmp_48 = 1;
+  wire [_tmp_48-1:0] _tmp_49;
+  assign _tmp_49 = th_matmul == 50;
+  reg [_tmp_48-1:0] __tmp_49_1;
+  reg signed [32-1:0] _tmp_50;
   reg signed [32-1:0] _th_matmul_v_28;
 
   always @(posedge CLK) begin
@@ -867,145 +868,146 @@ module blinkled
 
   always @(posedge CLK) begin
     if(RST) begin
-      _tmp_5 <= 0;
-      _tmp_3 <= 0;
       _tmp_6 <= 0;
-      _tmp_7 <= 0;
       _tmp_4 <= 0;
+      _tmp_7 <= 0;
+      _tmp_8 <= 0;
+      _tmp_5 <= 0;
       _ram_a_cond_0_1 <= 0;
-      __tmp_19_1 <= 0;
+      __tmp_20_1 <= 0;
     end else begin
       if(_ram_a_cond_0_1) begin
-        _tmp_7 <= 0;
-        _tmp_4 <= 0;
+        _tmp_8 <= 0;
+        _tmp_5 <= 0;
       end 
-      if(_myaxi_read_start && (_myaxi_read_op_sel == 1) && (_tmp_3 == 0)) begin
-        _tmp_5 <= _myaxi_read_local_addr - _myaxi_read_local_stride;
-        _tmp_3 <= _myaxi_read_size;
+      if(_myaxi_read_start && (_myaxi_read_op_sel == 1) && (_tmp_4 == 0)) begin
+        _tmp_6 <= _myaxi_read_local_addr - _myaxi_read_local_stride;
+        _tmp_4 <= _myaxi_read_size;
       end 
-      if(_dataflow__variable_ovalid_0 && ((_tmp_3 > 0) && !_tmp_4) && (_tmp_3 > 0)) begin
-        _tmp_5 <= _tmp_5 + _myaxi_read_local_stride;
-        _tmp_6 <= _dataflow__variable_odata_0;
-        _tmp_7 <= 1;
-        _tmp_3 <= _tmp_3 - 1;
+      if(_dataflow__variable_ovalid_0 && ((_tmp_4 > 0) && !_tmp_5) && (_tmp_4 > 0)) begin
+        _tmp_6 <= _tmp_6 + _myaxi_read_local_stride;
+        _tmp_7 <= _dataflow__variable_odata_0;
+        _tmp_8 <= 1;
+        _tmp_4 <= _tmp_4 - 1;
       end 
-      if(_dataflow__variable_ovalid_0 && ((_tmp_3 > 0) && !_tmp_4) && (_tmp_3 == 1)) begin
-        _tmp_4 <= 1;
+      if(_dataflow__variable_ovalid_0 && ((_tmp_4 > 0) && !_tmp_5) && (_tmp_4 == 1)) begin
+        _tmp_5 <= 1;
       end 
       _ram_a_cond_0_1 <= 1;
-      __tmp_19_1 <= _tmp_19;
+      __tmp_20_1 <= _tmp_20;
     end
   end
 
 
   always @(posedge CLK) begin
     if(RST) begin
-      _tmp_15 <= 0;
-      _tmp_13 <= 0;
       _tmp_16 <= 0;
-      _tmp_17 <= 0;
       _tmp_14 <= 0;
+      _tmp_17 <= 0;
+      _tmp_18 <= 0;
+      _tmp_15 <= 0;
       _ram_b_cond_0_1 <= 0;
-      __tmp_22_1 <= 0;
+      __tmp_23_1 <= 0;
     end else begin
       if(_ram_b_cond_0_1) begin
-        _tmp_17 <= 0;
-        _tmp_14 <= 0;
+        _tmp_18 <= 0;
+        _tmp_15 <= 0;
       end 
-      if(_myaxi_read_start && (_myaxi_read_op_sel == 2) && (_tmp_13 == 0)) begin
-        _tmp_15 <= _myaxi_read_local_addr - _myaxi_read_local_stride;
-        _tmp_13 <= _myaxi_read_size;
+      if(_myaxi_read_start && (_myaxi_read_op_sel == 2) && (_tmp_14 == 0)) begin
+        _tmp_16 <= _myaxi_read_local_addr - _myaxi_read_local_stride;
+        _tmp_14 <= _myaxi_read_size;
       end 
-      if(_dataflow__variable_ovalid_1 && ((_tmp_13 > 0) && !_tmp_14) && (_tmp_13 > 0)) begin
-        _tmp_15 <= _tmp_15 + _myaxi_read_local_stride;
-        _tmp_16 <= _dataflow__variable_odata_1;
-        _tmp_17 <= 1;
-        _tmp_13 <= _tmp_13 - 1;
+      if(_dataflow__variable_ovalid_1 && ((_tmp_14 > 0) && !_tmp_15) && (_tmp_14 > 0)) begin
+        _tmp_16 <= _tmp_16 + _myaxi_read_local_stride;
+        _tmp_17 <= _dataflow__variable_odata_1;
+        _tmp_18 <= 1;
+        _tmp_14 <= _tmp_14 - 1;
       end 
-      if(_dataflow__variable_ovalid_1 && ((_tmp_13 > 0) && !_tmp_14) && (_tmp_13 == 1)) begin
-        _tmp_14 <= 1;
+      if(_dataflow__variable_ovalid_1 && ((_tmp_14 > 0) && !_tmp_15) && (_tmp_14 == 1)) begin
+        _tmp_15 <= 1;
       end 
       _ram_b_cond_0_1 <= 1;
-      __tmp_22_1 <= _tmp_22;
+      __tmp_23_1 <= _tmp_23;
     end
   end
 
 
   always @(posedge CLK) begin
     if(RST) begin
-      _tmp_33 <= 0;
-      _tmp_25 <= 0;
-      _tmp_26 <= 0;
-      _tmp_31 <= 0;
-      _tmp_32 <= 0;
-      _tmp_30 <= 0;
-      _tmp_35 <= 0;
       _tmp_34 <= 0;
-      _tmp_44 <= 0;
-      _tmp_42 <= 0;
+      _tmp_26 <= 0;
+      _tmp_27 <= 0;
+      _tmp_32 <= 0;
+      _tmp_33 <= 0;
+      _tmp_31 <= 0;
+      _tmp_36 <= 0;
+      _tmp_35 <= 0;
       _tmp_45 <= 0;
-      _tmp_46 <= 0;
       _tmp_43 <= 0;
+      _tmp_46 <= 0;
+      _tmp_47 <= 0;
+      _tmp_44 <= 0;
       _ram_c_cond_0_1 <= 0;
-      __tmp_48_1 <= 0;
+      __tmp_49_1 <= 0;
     end else begin
       if(_ram_c_cond_0_1) begin
-        _tmp_46 <= 0;
-        _tmp_43 <= 0;
+        _tmp_47 <= 0;
+        _tmp_44 <= 0;
       end 
-      if((_tmp_27 || !_tmp_25) && (_tmp_28 || !_tmp_26) && _tmp_31) begin
-        _tmp_33 <= 0;
-        _tmp_25 <= 0;
+      if((_tmp_28 || !_tmp_26) && (_tmp_29 || !_tmp_27) && _tmp_32) begin
+        _tmp_34 <= 0;
         _tmp_26 <= 0;
-        _tmp_31 <= 0;
+        _tmp_27 <= 0;
+        _tmp_32 <= 0;
       end 
-      if((_tmp_27 || !_tmp_25) && (_tmp_28 || !_tmp_26) && _tmp_30) begin
-        _tmp_25 <= 1;
+      if((_tmp_28 || !_tmp_26) && (_tmp_29 || !_tmp_27) && _tmp_31) begin
         _tmp_26 <= 1;
-        _tmp_33 <= _tmp_32;
-        _tmp_32 <= 0;
-        _tmp_30 <= 0;
-        _tmp_31 <= 1;
-      end 
-      if(_myaxi_write_start && (_myaxi_write_op_sel == 1) && (_tmp_34 == 0) && !_tmp_32 && !_tmp_33) begin
-        _tmp_35 <= _myaxi_write_local_addr;
-        _tmp_34 <= _myaxi_write_size - 1;
-        _tmp_30 <= 1;
-        _tmp_32 <= _myaxi_write_size == 1;
-      end 
-      if((_tmp_27 || !_tmp_25) && (_tmp_28 || !_tmp_26) && (_tmp_34 > 0)) begin
-        _tmp_35 <= _tmp_35 + _myaxi_write_local_stride;
-        _tmp_34 <= _tmp_34 - 1;
-        _tmp_30 <= 1;
-        _tmp_32 <= 0;
-      end 
-      if((_tmp_27 || !_tmp_25) && (_tmp_28 || !_tmp_26) && (_tmp_34 == 1)) begin
+        _tmp_27 <= 1;
+        _tmp_34 <= _tmp_33;
+        _tmp_33 <= 0;
+        _tmp_31 <= 0;
         _tmp_32 <= 1;
       end 
-      if(_myaxi_read_start && (_myaxi_read_op_sel == 3) && (_tmp_42 == 0)) begin
-        _tmp_44 <= _myaxi_read_local_addr - _myaxi_read_local_stride;
-        _tmp_42 <= _myaxi_read_size;
+      if(_myaxi_write_start && (_myaxi_write_op_sel == 1) && (_tmp_35 == 0) && !_tmp_33 && !_tmp_34) begin
+        _tmp_36 <= _myaxi_write_local_addr;
+        _tmp_35 <= _myaxi_write_size - 1;
+        _tmp_31 <= 1;
+        _tmp_33 <= _myaxi_write_size == 1;
       end 
-      if(_dataflow__variable_ovalid_4 && ((_tmp_42 > 0) && !_tmp_43) && (_tmp_42 > 0)) begin
-        _tmp_44 <= _tmp_44 + _myaxi_read_local_stride;
-        _tmp_45 <= _dataflow__variable_odata_4;
-        _tmp_46 <= 1;
-        _tmp_42 <= _tmp_42 - 1;
+      if((_tmp_28 || !_tmp_26) && (_tmp_29 || !_tmp_27) && (_tmp_35 > 0)) begin
+        _tmp_36 <= _tmp_36 + _myaxi_write_local_stride;
+        _tmp_35 <= _tmp_35 - 1;
+        _tmp_31 <= 1;
+        _tmp_33 <= 0;
       end 
-      if(_dataflow__variable_ovalid_4 && ((_tmp_42 > 0) && !_tmp_43) && (_tmp_42 == 1)) begin
-        _tmp_43 <= 1;
+      if((_tmp_28 || !_tmp_26) && (_tmp_29 || !_tmp_27) && (_tmp_35 == 1)) begin
+        _tmp_33 <= 1;
+      end 
+      if(_myaxi_read_start && (_myaxi_read_op_sel == 3) && (_tmp_43 == 0)) begin
+        _tmp_45 <= _myaxi_read_local_addr - _myaxi_read_local_stride;
+        _tmp_43 <= _myaxi_read_size;
+      end 
+      if(_dataflow__variable_ovalid_4 && ((_tmp_43 > 0) && !_tmp_44) && (_tmp_43 > 0)) begin
+        _tmp_45 <= _tmp_45 + _myaxi_read_local_stride;
+        _tmp_46 <= _dataflow__variable_odata_4;
+        _tmp_47 <= 1;
+        _tmp_43 <= _tmp_43 - 1;
+      end 
+      if(_dataflow__variable_ovalid_4 && ((_tmp_43 > 0) && !_tmp_44) && (_tmp_43 == 1)) begin
+        _tmp_44 <= 1;
       end 
       _ram_c_cond_0_1 <= 1;
-      __tmp_48_1 <= _tmp_48;
+      __tmp_49_1 <= _tmp_49;
     end
   end
 
-  assign _dataflow__variable_odata_2 = _tmp_29;
-  assign _dataflow__variable_ovalid_2 = _tmp_25;
-  assign _tmp_27 = 1 && _dataflow__variable_oready_2;
+  assign _dataflow__variable_odata_2 = _tmp_30;
+  assign _dataflow__variable_ovalid_2 = _tmp_26;
+  assign _tmp_28 = 1 && _dataflow__variable_oready_2;
 
   always @(posedge CLK) begin
     if(RST) begin
+      outstanding_wreq_count_0 <= 0;
       _myaxi_read_start <= 0;
       _myaxi_write_start <= 0;
       _myaxi_ram_a_0_read_start <= 0;
@@ -1023,7 +1025,7 @@ module blinkled
       myaxi_araddr <= 0;
       myaxi_arlen <= 0;
       myaxi_arvalid <= 0;
-      _tmp_8 <= 0;
+      counter_9 <= 0;
       _myaxi_cond_0_1 <= 0;
       _myaxi_ram_b_0_read_start <= 0;
       _myaxi_ram_b_0_read_op_sel <= 0;
@@ -1046,13 +1048,13 @@ module blinkled
       myaxi_awaddr <= 0;
       myaxi_awlen <= 0;
       myaxi_awvalid <= 0;
-      _tmp_36 <= 0;
+      counter_37 <= 0;
       _myaxi_cond_1_1 <= 0;
       myaxi_wdata <= 0;
       myaxi_wvalid <= 0;
       myaxi_wlast <= 0;
       myaxi_wstrb <= 0;
-      _tmp_37 <= 0;
+      last_38 <= 0;
       _myaxi_cond_2_1 <= 0;
       _myaxi_ram_c_0_read_start <= 0;
       _myaxi_ram_c_0_read_op_sel <= 0;
@@ -1070,12 +1072,18 @@ module blinkled
       if(_myaxi_cond_2_1) begin
         myaxi_wvalid <= 0;
         myaxi_wlast <= 0;
-        _tmp_37 <= 0;
+        last_38 <= 0;
+      end 
+      if(myaxi_awvalid && myaxi_awready && !(myaxi_bvalid && myaxi_bready)) begin
+        outstanding_wreq_count_0 <= outstanding_wreq_count_0 + 1;
+      end 
+      if(!(myaxi_awvalid && myaxi_awready) && (myaxi_bvalid && myaxi_bready) && (outstanding_wreq_count_0 > 0)) begin
+        outstanding_wreq_count_0 <= outstanding_wreq_count_0 - 1;
       end 
       _myaxi_read_start <= 0;
       _myaxi_write_start <= 0;
       _myaxi_ram_a_0_read_start <= 0;
-      if(axim_flag_0) begin
+      if(axim_flag_1) begin
         _myaxi_ram_a_0_read_start <= 1;
         _myaxi_ram_a_0_read_op_sel <= 1;
         _myaxi_ram_a_0_read_local_addr <= 0;
@@ -1094,24 +1102,24 @@ module blinkled
         _myaxi_read_size <= _myaxi_ram_a_0_read_size;
         _myaxi_read_local_stride <= _myaxi_ram_a_0_read_local_stride;
       end 
-      if((_myaxi_read_fsm == 2) && ((myaxi_arready || !myaxi_arvalid) && (_tmp_8 == 0))) begin
+      if((_myaxi_read_fsm == 2) && ((myaxi_arready || !myaxi_arvalid) && (counter_9 == 0))) begin
         myaxi_araddr <= _myaxi_read_cur_global_addr;
         myaxi_arlen <= _myaxi_read_cur_size - 1;
         myaxi_arvalid <= 1;
-        _tmp_8 <= _myaxi_read_cur_size;
+        counter_9 <= _myaxi_read_cur_size;
       end 
       _myaxi_cond_0_1 <= 1;
       if(myaxi_arvalid && !myaxi_arready) begin
         myaxi_arvalid <= myaxi_arvalid;
       end 
-      if(myaxi_rready && myaxi_rvalid && (_tmp_8 > 0)) begin
-        _tmp_8 <= _tmp_8 - 1;
+      if(myaxi_rready && myaxi_rvalid && (counter_9 > 0)) begin
+        counter_9 <= counter_9 - 1;
       end 
-      if(axim_flag_9) begin
+      if(axim_flag_10) begin
         _myaxi_read_idle <= 1;
       end 
       _myaxi_ram_b_0_read_start <= 0;
-      if(axim_flag_10) begin
+      if(axim_flag_11) begin
         _myaxi_ram_b_0_read_start <= 1;
         _myaxi_ram_b_0_read_op_sel <= 2;
         _myaxi_ram_b_0_read_local_addr <= 0;
@@ -1131,7 +1139,7 @@ module blinkled
         _myaxi_read_local_stride <= _myaxi_ram_b_0_read_local_stride;
       end 
       _myaxi_ram_c_0_write_start <= 0;
-      if(axim_flag_24) begin
+      if(axim_flag_25) begin
         _myaxi_ram_c_0_write_start <= 1;
         _myaxi_ram_c_0_write_op_sel <= 1;
         _myaxi_ram_c_0_write_local_addr <= 0;
@@ -1150,41 +1158,41 @@ module blinkled
         _myaxi_write_size <= _myaxi_ram_c_0_write_size;
         _myaxi_write_local_stride <= _myaxi_ram_c_0_write_local_stride;
       end 
-      if((_myaxi_write_fsm == 2) && ((myaxi_awready || !myaxi_awvalid) && (_tmp_36 == 0))) begin
+      if((_myaxi_write_fsm == 2) && ((myaxi_awready || !myaxi_awvalid) && (counter_37 == 0))) begin
         myaxi_awaddr <= _myaxi_write_cur_global_addr;
         myaxi_awlen <= _myaxi_write_cur_size - 1;
         myaxi_awvalid <= 1;
-        _tmp_36 <= _myaxi_write_cur_size;
+        counter_37 <= _myaxi_write_cur_size;
       end 
-      if((_myaxi_write_fsm == 2) && ((myaxi_awready || !myaxi_awvalid) && (_tmp_36 == 0)) && (_myaxi_write_cur_size == 0)) begin
+      if((_myaxi_write_fsm == 2) && ((myaxi_awready || !myaxi_awvalid) && (counter_37 == 0)) && (_myaxi_write_cur_size == 0)) begin
         myaxi_awvalid <= 0;
       end 
       _myaxi_cond_1_1 <= 1;
       if(myaxi_awvalid && !myaxi_awready) begin
         myaxi_awvalid <= myaxi_awvalid;
       end 
-      if(_dataflow__variable_ovalid_2 && ((_myaxi_write_fsm == 3) && (_myaxi_write_op_sel == 1) && ((_tmp_36 > 0) && (myaxi_wready || !myaxi_wvalid))) && ((_tmp_36 > 0) && (myaxi_wready || !myaxi_wvalid) && (_tmp_36 > 0))) begin
+      if(_dataflow__variable_ovalid_2 && ((_myaxi_write_fsm == 3) && (_myaxi_write_op_sel == 1) && ((counter_37 > 0) && (myaxi_wready || !myaxi_wvalid))) && ((counter_37 > 0) && (myaxi_wready || !myaxi_wvalid) && (counter_37 > 0))) begin
         myaxi_wdata <= _dataflow__variable_odata_2;
         myaxi_wvalid <= 1;
         myaxi_wlast <= 0;
         myaxi_wstrb <= { 4{ 1'd1 } };
-        _tmp_36 <= _tmp_36 - 1;
+        counter_37 <= counter_37 - 1;
       end 
-      if(_dataflow__variable_ovalid_2 && ((_myaxi_write_fsm == 3) && (_myaxi_write_op_sel == 1) && ((_tmp_36 > 0) && (myaxi_wready || !myaxi_wvalid))) && ((_tmp_36 > 0) && (myaxi_wready || !myaxi_wvalid) && (_tmp_36 > 0)) && (_tmp_36 == 1)) begin
+      if(_dataflow__variable_ovalid_2 && ((_myaxi_write_fsm == 3) && (_myaxi_write_op_sel == 1) && ((counter_37 > 0) && (myaxi_wready || !myaxi_wvalid))) && ((counter_37 > 0) && (myaxi_wready || !myaxi_wvalid) && (counter_37 > 0)) && (counter_37 == 1)) begin
         myaxi_wlast <= 1;
-        _tmp_37 <= 1;
+        last_38 <= 1;
       end 
       _myaxi_cond_2_1 <= 1;
       if(myaxi_wvalid && !myaxi_wready) begin
         myaxi_wvalid <= myaxi_wvalid;
         myaxi_wlast <= myaxi_wlast;
-        _tmp_37 <= _tmp_37;
+        last_38 <= last_38;
       end 
-      if(axim_flag_38) begin
+      if(axim_flag_39) begin
         _myaxi_write_idle <= 1;
       end 
       _myaxi_ram_c_0_read_start <= 0;
-      if(axim_flag_39) begin
+      if(axim_flag_40) begin
         _myaxi_ram_c_0_read_start <= 1;
         _myaxi_ram_c_0_read_op_sel <= 3;
         _myaxi_ram_c_0_read_local_addr <= 0;
@@ -1206,12 +1214,12 @@ module blinkled
     end
   end
 
-  assign _dataflow__variable_odata_0 = _wdata_1;
-  assign _dataflow__variable_ovalid_0 = _wvalid_2;
-  assign _dataflow__variable_odata_1 = _wdata_11;
-  assign _dataflow__variable_ovalid_1 = _wvalid_12;
-  assign _dataflow__variable_odata_4 = _wdata_40;
-  assign _dataflow__variable_ovalid_4 = _wvalid_41;
+  assign _dataflow__variable_odata_0 = _wdata_2;
+  assign _dataflow__variable_ovalid_0 = _wvalid_3;
+  assign _dataflow__variable_odata_1 = _wdata_12;
+  assign _dataflow__variable_ovalid_1 = _wvalid_13;
+  assign _dataflow__variable_odata_4 = _wdata_41;
+  assign _dataflow__variable_ovalid_4 = _wvalid_42;
   localparam th_matmul_1 = 1;
   localparam th_matmul_2 = 2;
   localparam th_matmul_3 = 3;
@@ -1295,19 +1303,19 @@ module blinkled
       _th_matmul_a_addr_9 <= 0;
       _th_matmul_c_addr_10 <= 0;
       _th_matmul_i_11 <= 0;
-      axim_flag_0 <= 0;
+      axim_flag_1 <= 0;
       _th_matmul_cond_6_0_1 <= 0;
       _th_matmul_b_addr_12 <= 0;
       _th_matmul_j_13 <= 0;
-      axim_flag_10 <= 0;
+      axim_flag_11 <= 0;
       _th_matmul_cond_13_1_1 <= 0;
       _th_matmul_sum_14 <= 0;
       _th_matmul_k_15 <= 0;
-      _tmp_20 <= 0;
+      _tmp_21 <= 0;
       _th_matmul_x_16 <= 0;
-      _tmp_23 <= 0;
+      _tmp_24 <= 0;
       _th_matmul_y_17 <= 0;
-      axim_flag_24 <= 0;
+      axim_flag_25 <= 0;
       _th_matmul_cond_29_2_1 <= 0;
       _th_matmul_end_time_18 <= 0;
       _th_matmul_time_19 <= 0;
@@ -1318,32 +1326,32 @@ module blinkled
       _th_matmul_all_ok_24 <= 0;
       _th_matmul_c_addr_25 <= 0;
       _th_matmul_i_26 <= 0;
-      axim_flag_39 <= 0;
+      axim_flag_40 <= 0;
       _th_matmul_cond_44_3_1 <= 0;
       _th_matmul_j_27 <= 0;
-      _tmp_49 <= 0;
+      _tmp_50 <= 0;
       _th_matmul_v_28 <= 0;
     end else begin
       _d1_th_matmul <= th_matmul;
       case(_d1_th_matmul)
         th_matmul_6: begin
           if(_th_matmul_cond_6_0_1) begin
-            axim_flag_0 <= 0;
+            axim_flag_1 <= 0;
           end 
         end
         th_matmul_13: begin
           if(_th_matmul_cond_13_1_1) begin
-            axim_flag_10 <= 0;
+            axim_flag_11 <= 0;
           end 
         end
         th_matmul_29: begin
           if(_th_matmul_cond_29_2_1) begin
-            axim_flag_24 <= 0;
+            axim_flag_25 <= 0;
           end 
         end
         th_matmul_44: begin
           if(_th_matmul_cond_44_3_1) begin
-            axim_flag_39 <= 0;
+            axim_flag_40 <= 0;
           end 
         end
       endcase
@@ -1383,7 +1391,7 @@ module blinkled
           end
         end
         th_matmul_6: begin
-          axim_flag_0 <= 1;
+          axim_flag_1 <= 1;
           _th_matmul_cond_6_0_1 <= 1;
           th_matmul <= th_matmul_7;
         end
@@ -1414,7 +1422,7 @@ module blinkled
           end
         end
         th_matmul_13: begin
-          axim_flag_10 <= 1;
+          axim_flag_11 <= 1;
           _th_matmul_cond_13_1_1 <= 1;
           th_matmul <= th_matmul_14;
         end
@@ -1445,27 +1453,27 @@ module blinkled
           end
         end
         th_matmul_20: begin
-          if(__tmp_19_1) begin
-            _tmp_20 <= ram_a_0_rdata;
+          if(__tmp_20_1) begin
+            _tmp_21 <= ram_a_0_rdata;
           end 
-          if(__tmp_19_1) begin
+          if(__tmp_20_1) begin
             th_matmul <= th_matmul_21;
           end 
         end
         th_matmul_21: begin
-          _th_matmul_x_16 <= _tmp_20;
+          _th_matmul_x_16 <= _tmp_21;
           th_matmul <= th_matmul_22;
         end
         th_matmul_22: begin
-          if(__tmp_22_1) begin
-            _tmp_23 <= ram_b_0_rdata;
+          if(__tmp_23_1) begin
+            _tmp_24 <= ram_b_0_rdata;
           end 
-          if(__tmp_22_1) begin
+          if(__tmp_23_1) begin
             th_matmul <= th_matmul_23;
           end 
         end
         th_matmul_23: begin
-          _th_matmul_y_17 <= _tmp_23;
+          _th_matmul_y_17 <= _tmp_24;
           th_matmul <= th_matmul_24;
         end
         th_matmul_24: begin
@@ -1488,7 +1496,7 @@ module blinkled
           th_matmul <= th_matmul_12;
         end
         th_matmul_29: begin
-          axim_flag_24 <= 1;
+          axim_flag_25 <= 1;
           _th_matmul_cond_29_2_1 <= 1;
           th_matmul <= th_matmul_30;
         end
@@ -1499,7 +1507,7 @@ module blinkled
           th_matmul <= th_matmul_32;
         end
         th_matmul_32: begin
-          if(_myaxi_write_idle) begin
+          if(_myaxi_write_idle && (outstanding_wreq_count_0 == 0)) begin
             th_matmul <= th_matmul_33;
           end 
         end
@@ -1554,7 +1562,7 @@ module blinkled
           end
         end
         th_matmul_44: begin
-          axim_flag_39 <= 1;
+          axim_flag_40 <= 1;
           _th_matmul_cond_44_3_1 <= 1;
           th_matmul <= th_matmul_45;
         end
@@ -1581,15 +1589,15 @@ module blinkled
           end
         end
         th_matmul_50: begin
-          if(__tmp_48_1) begin
-            _tmp_49 <= ram_c_0_rdata;
+          if(__tmp_49_1) begin
+            _tmp_50 <= ram_c_0_rdata;
           end 
-          if(__tmp_48_1) begin
+          if(__tmp_49_1) begin
             th_matmul <= th_matmul_51;
           end 
         end
         th_matmul_51: begin
-          _th_matmul_v_28 <= _tmp_49;
+          _th_matmul_v_28 <= _tmp_50;
           th_matmul <= th_matmul_52;
         end
         th_matmul_52: begin
@@ -1674,33 +1682,33 @@ module blinkled
       _myaxi_read_rest_size <= 0;
       _myaxi_read_cur_size <= 0;
       __myaxi_read_fsm_cond_3_0_1 <= 0;
-      _wvalid_2 <= 0;
-      _wdata_1 <= 0;
-      axim_flag_9 <= 0;
+      _wvalid_3 <= 0;
+      _wdata_2 <= 0;
+      axim_flag_10 <= 0;
       __myaxi_read_fsm_cond_4_1_1 <= 0;
       __myaxi_read_fsm_cond_3_2_1 <= 0;
-      _wvalid_12 <= 0;
-      _wdata_11 <= 0;
+      _wvalid_13 <= 0;
+      _wdata_12 <= 0;
       __myaxi_read_fsm_cond_3_3_1 <= 0;
-      _wvalid_41 <= 0;
-      _wdata_40 <= 0;
+      _wvalid_42 <= 0;
+      _wdata_41 <= 0;
     end else begin
       _d1__myaxi_read_fsm <= _myaxi_read_fsm;
       case(_d1__myaxi_read_fsm)
         _myaxi_read_fsm_3: begin
           if(__myaxi_read_fsm_cond_3_0_1) begin
-            _wvalid_2 <= 0;
+            _wvalid_3 <= 0;
           end 
           if(__myaxi_read_fsm_cond_3_2_1) begin
-            _wvalid_12 <= 0;
+            _wvalid_13 <= 0;
           end 
           if(__myaxi_read_fsm_cond_3_3_1) begin
-            _wvalid_41 <= 0;
+            _wvalid_42 <= 0;
           end 
         end
         _myaxi_read_fsm_4: begin
           if(__myaxi_read_fsm_cond_4_1_1) begin
-            axim_flag_9 <= 0;
+            axim_flag_10 <= 0;
           end 
         end
       endcase
@@ -1744,21 +1752,21 @@ module blinkled
         _myaxi_read_fsm_3: begin
           __myaxi_read_fsm_cond_3_0_1 <= 1;
           if(myaxi_rready && myaxi_rvalid && (_myaxi_read_op_sel == 1)) begin
-            _wdata_1 <= myaxi_rdata;
-            _wvalid_2 <= 1;
+            _wdata_2 <= myaxi_rdata;
+            _wvalid_3 <= 1;
           end 
           if(myaxi_rready && myaxi_rvalid && myaxi_rlast) begin
             _myaxi_read_cur_global_addr <= _myaxi_read_cur_global_addr + (_myaxi_read_cur_size << 2);
           end 
           __myaxi_read_fsm_cond_3_2_1 <= 1;
           if(myaxi_rready && myaxi_rvalid && (_myaxi_read_op_sel == 2)) begin
-            _wdata_11 <= myaxi_rdata;
-            _wvalid_12 <= 1;
+            _wdata_12 <= myaxi_rdata;
+            _wvalid_13 <= 1;
           end 
           __myaxi_read_fsm_cond_3_3_1 <= 1;
           if(myaxi_rready && myaxi_rvalid && (_myaxi_read_op_sel == 3)) begin
-            _wdata_40 <= myaxi_rdata;
-            _wvalid_41 <= 1;
+            _wdata_41 <= myaxi_rdata;
+            _wvalid_42 <= 1;
           end 
           if(myaxi_rready && myaxi_rvalid && myaxi_rlast && (_myaxi_read_rest_size > 0)) begin
             _myaxi_read_fsm <= _myaxi_read_fsm_1;
@@ -1768,7 +1776,7 @@ module blinkled
           end 
         end
         _myaxi_read_fsm_4: begin
-          axim_flag_9 <= 1;
+          axim_flag_10 <= 1;
           __myaxi_read_fsm_cond_4_1_1 <= 1;
           _myaxi_read_fsm <= _myaxi_read_fsm_5;
         end
@@ -1792,14 +1800,14 @@ module blinkled
       _myaxi_write_cur_global_addr <= 0;
       _myaxi_write_rest_size <= 0;
       _myaxi_write_cur_size <= 0;
-      axim_flag_38 <= 0;
+      axim_flag_39 <= 0;
       __myaxi_write_fsm_cond_4_0_1 <= 0;
     end else begin
       _d1__myaxi_write_fsm <= _myaxi_write_fsm;
       case(_d1__myaxi_write_fsm)
         _myaxi_write_fsm_4: begin
           if(__myaxi_write_fsm_cond_4_0_1) begin
-            axim_flag_38 <= 0;
+            axim_flag_39 <= 0;
           end 
         end
       endcase
@@ -1846,7 +1854,7 @@ module blinkled
           end 
         end
         _myaxi_write_fsm_4: begin
-          axim_flag_38 <= 1;
+          axim_flag_39 <= 1;
           __myaxi_write_fsm_cond_4_0_1 <= 1;
           _myaxi_write_fsm <= _myaxi_write_fsm_5;
         end
