@@ -93,183 +93,184 @@ module test;
   assign _axi_arprot = 0;
   assign _axi_arqos = 0;
   assign _axi_aruser = 0;
-  wire [32-1:0] _tmp_0;
-  assign _tmp_0 = _axi_awaddr;
+  reg [32-1:0] outstanding_wreq_count_0;
+  wire [32-1:0] _tmp_1;
+  assign _tmp_1 = _axi_awaddr;
 
   always @(*) begin
-    myaxi_awaddr = _tmp_0;
+    myaxi_awaddr = _tmp_1;
   end
 
-  wire [8-1:0] _tmp_1;
-  assign _tmp_1 = _axi_awlen;
+  wire [8-1:0] _tmp_2;
+  assign _tmp_2 = _axi_awlen;
 
   always @(*) begin
-    myaxi_awlen = _tmp_1;
+    myaxi_awlen = _tmp_2;
   end
 
-  wire [3-1:0] _tmp_2;
-  assign _tmp_2 = _axi_awsize;
+  wire [3-1:0] _tmp_3;
+  assign _tmp_3 = _axi_awsize;
 
   always @(*) begin
-    myaxi_awsize = _tmp_2;
+    myaxi_awsize = _tmp_3;
   end
 
-  wire [2-1:0] _tmp_3;
-  assign _tmp_3 = _axi_awburst;
+  wire [2-1:0] _tmp_4;
+  assign _tmp_4 = _axi_awburst;
 
   always @(*) begin
-    myaxi_awburst = _tmp_3;
+    myaxi_awburst = _tmp_4;
   end
 
-  wire [1-1:0] _tmp_4;
-  assign _tmp_4 = _axi_awlock;
+  wire [1-1:0] _tmp_5;
+  assign _tmp_5 = _axi_awlock;
 
   always @(*) begin
-    myaxi_awlock = _tmp_4;
+    myaxi_awlock = _tmp_5;
   end
 
-  wire [4-1:0] _tmp_5;
-  assign _tmp_5 = _axi_awcache;
+  wire [4-1:0] _tmp_6;
+  assign _tmp_6 = _axi_awcache;
 
   always @(*) begin
-    myaxi_awcache = _tmp_5;
+    myaxi_awcache = _tmp_6;
   end
 
-  wire [3-1:0] _tmp_6;
-  assign _tmp_6 = _axi_awprot;
+  wire [3-1:0] _tmp_7;
+  assign _tmp_7 = _axi_awprot;
 
   always @(*) begin
-    myaxi_awprot = _tmp_6;
+    myaxi_awprot = _tmp_7;
   end
 
-  wire [4-1:0] _tmp_7;
-  assign _tmp_7 = _axi_awqos;
+  wire [4-1:0] _tmp_8;
+  assign _tmp_8 = _axi_awqos;
 
   always @(*) begin
-    myaxi_awqos = _tmp_7;
+    myaxi_awqos = _tmp_8;
   end
 
-  wire [2-1:0] _tmp_8;
-  assign _tmp_8 = _axi_awuser;
+  wire [2-1:0] _tmp_9;
+  assign _tmp_9 = _axi_awuser;
 
   always @(*) begin
-    myaxi_awuser = _tmp_8;
+    myaxi_awuser = _tmp_9;
   end
 
-  wire _tmp_9;
-  assign _tmp_9 = _axi_awvalid;
+  wire _tmp_10;
+  assign _tmp_10 = _axi_awvalid;
 
   always @(*) begin
-    myaxi_awvalid = _tmp_9;
+    myaxi_awvalid = _tmp_10;
   end
 
   assign _axi_awready = myaxi_awready;
-  wire [32-1:0] _tmp_10;
-  assign _tmp_10 = _axi_wdata;
+  wire [32-1:0] _tmp_11;
+  assign _tmp_11 = _axi_wdata;
 
   always @(*) begin
-    myaxi_wdata = _tmp_10;
+    myaxi_wdata = _tmp_11;
   end
 
-  wire [4-1:0] _tmp_11;
-  assign _tmp_11 = _axi_wstrb;
+  wire [4-1:0] _tmp_12;
+  assign _tmp_12 = _axi_wstrb;
 
   always @(*) begin
-    myaxi_wstrb = _tmp_11;
-  end
-
-  wire _tmp_12;
-  assign _tmp_12 = _axi_wlast;
-
-  always @(*) begin
-    myaxi_wlast = _tmp_12;
+    myaxi_wstrb = _tmp_12;
   end
 
   wire _tmp_13;
-  assign _tmp_13 = _axi_wvalid;
+  assign _tmp_13 = _axi_wlast;
 
   always @(*) begin
-    myaxi_wvalid = _tmp_13;
+    myaxi_wlast = _tmp_13;
+  end
+
+  wire _tmp_14;
+  assign _tmp_14 = _axi_wvalid;
+
+  always @(*) begin
+    myaxi_wvalid = _tmp_14;
   end
 
   assign _axi_wready = myaxi_wready;
   assign _axi_bresp = myaxi_bresp;
   assign _axi_bvalid = myaxi_bvalid;
-  wire _tmp_14;
-  assign _tmp_14 = _axi_bready;
+  wire _tmp_15;
+  assign _tmp_15 = _axi_bready;
 
   always @(*) begin
-    myaxi_bready = _tmp_14;
+    myaxi_bready = _tmp_15;
   end
 
-  wire [32-1:0] _tmp_15;
-  assign _tmp_15 = _axi_araddr;
+  wire [32-1:0] _tmp_16;
+  assign _tmp_16 = _axi_araddr;
 
   always @(*) begin
-    myaxi_araddr = _tmp_15;
+    myaxi_araddr = _tmp_16;
   end
 
-  wire [8-1:0] _tmp_16;
-  assign _tmp_16 = _axi_arlen;
+  wire [8-1:0] _tmp_17;
+  assign _tmp_17 = _axi_arlen;
 
   always @(*) begin
-    myaxi_arlen = _tmp_16;
+    myaxi_arlen = _tmp_17;
   end
 
-  wire [3-1:0] _tmp_17;
-  assign _tmp_17 = _axi_arsize;
+  wire [3-1:0] _tmp_18;
+  assign _tmp_18 = _axi_arsize;
 
   always @(*) begin
-    myaxi_arsize = _tmp_17;
+    myaxi_arsize = _tmp_18;
   end
 
-  wire [2-1:0] _tmp_18;
-  assign _tmp_18 = _axi_arburst;
+  wire [2-1:0] _tmp_19;
+  assign _tmp_19 = _axi_arburst;
 
   always @(*) begin
-    myaxi_arburst = _tmp_18;
+    myaxi_arburst = _tmp_19;
   end
 
-  wire [1-1:0] _tmp_19;
-  assign _tmp_19 = _axi_arlock;
+  wire [1-1:0] _tmp_20;
+  assign _tmp_20 = _axi_arlock;
 
   always @(*) begin
-    myaxi_arlock = _tmp_19;
+    myaxi_arlock = _tmp_20;
   end
 
-  wire [4-1:0] _tmp_20;
-  assign _tmp_20 = _axi_arcache;
+  wire [4-1:0] _tmp_21;
+  assign _tmp_21 = _axi_arcache;
 
   always @(*) begin
-    myaxi_arcache = _tmp_20;
+    myaxi_arcache = _tmp_21;
   end
 
-  wire [3-1:0] _tmp_21;
-  assign _tmp_21 = _axi_arprot;
+  wire [3-1:0] _tmp_22;
+  assign _tmp_22 = _axi_arprot;
 
   always @(*) begin
-    myaxi_arprot = _tmp_21;
+    myaxi_arprot = _tmp_22;
   end
 
-  wire [4-1:0] _tmp_22;
-  assign _tmp_22 = _axi_arqos;
+  wire [4-1:0] _tmp_23;
+  assign _tmp_23 = _axi_arqos;
 
   always @(*) begin
-    myaxi_arqos = _tmp_22;
+    myaxi_arqos = _tmp_23;
   end
 
-  wire [2-1:0] _tmp_23;
-  assign _tmp_23 = _axi_aruser;
+  wire [2-1:0] _tmp_24;
+  assign _tmp_24 = _axi_aruser;
 
   always @(*) begin
-    myaxi_aruser = _tmp_23;
+    myaxi_aruser = _tmp_24;
   end
 
-  wire _tmp_24;
-  assign _tmp_24 = _axi_arvalid;
+  wire _tmp_25;
+  assign _tmp_25 = _axi_arvalid;
 
   always @(*) begin
-    myaxi_arvalid = _tmp_24;
+    myaxi_arvalid = _tmp_25;
   end
 
   assign _axi_arready = myaxi_arready;
@@ -277,19 +278,19 @@ module test;
   assign _axi_rresp = myaxi_rresp;
   assign _axi_rlast = myaxi_rlast;
   assign _axi_rvalid = myaxi_rvalid;
-  wire _tmp_25;
-  assign _tmp_25 = _axi_rready;
+  wire _tmp_26;
+  assign _tmp_26 = _axi_rready;
 
   always @(*) begin
-    myaxi_rready = _tmp_25;
+    myaxi_rready = _tmp_26;
   end
 
   reg [32-1:0] fsm;
   localparam fsm_init = 0;
   reg [32-1:0] sum;
-  reg [9-1:0] _tmp_26;
+  reg [9-1:0] counter_27;
   reg __axi_cond_0_1;
-  reg [9-1:0] _tmp_27;
+  reg [9-1:0] counter_28;
   reg __axi_cond_1_1;
   assign _axi_rready = (fsm == 1) || (fsm == 3);
 
@@ -356,11 +357,12 @@ module test;
     _axi_araddr = 0;
     _axi_arlen = 0;
     _axi_arvalid = 0;
+    outstanding_wreq_count_0 = 0;
     fsm = fsm_init;
     sum = 0;
-    _tmp_26 = 0;
+    counter_27 = 0;
     __axi_cond_0_1 = 0;
-    _tmp_27 = 0;
+    counter_28 = 0;
     __axi_cond_1_1 = 0;
     #100;
     RST = 1;
@@ -373,6 +375,7 @@ module test;
 
   always @(posedge CLK) begin
     if(RST) begin
+      outstanding_wreq_count_0 <= 0;
       _axi_awaddr <= 0;
       _axi_awlen <= 0;
       _axi_awvalid <= 0;
@@ -383,9 +386,9 @@ module test;
       _axi_araddr <= 0;
       _axi_arlen <= 0;
       _axi_arvalid <= 0;
-      _tmp_26 <= 0;
+      counter_27 <= 0;
       __axi_cond_0_1 <= 0;
-      _tmp_27 <= 0;
+      counter_28 <= 0;
       __axi_cond_1_1 <= 0;
     end else begin
       if(__axi_cond_0_1) begin
@@ -394,6 +397,12 @@ module test;
       if(__axi_cond_1_1) begin
         _axi_arvalid <= 0;
       end 
+      if(_axi_awvalid && _axi_awready && !(_axi_bvalid && _axi_bready)) begin
+        outstanding_wreq_count_0 <= outstanding_wreq_count_0 + 1;
+      end 
+      if(!(_axi_awvalid && _axi_awready) && (_axi_bvalid && _axi_bready) && (outstanding_wreq_count_0 > 0)) begin
+        outstanding_wreq_count_0 <= outstanding_wreq_count_0 - 1;
+      end 
       _axi_awaddr <= 0;
       _axi_awlen <= 0;
       _axi_awvalid <= 0;
@@ -401,31 +410,31 @@ module test;
       _axi_wstrb <= 0;
       _axi_wlast <= 0;
       _axi_wvalid <= 0;
-      if((fsm == 0) && ((_axi_arready || !_axi_arvalid) && (_tmp_26 == 0))) begin
+      if((fsm == 0) && ((_axi_arready || !_axi_arvalid) && (counter_27 == 0))) begin
         _axi_araddr <= 1024;
         _axi_arlen <= 63;
         _axi_arvalid <= 1;
-        _tmp_26 <= 64;
+        counter_27 <= 64;
       end 
       __axi_cond_0_1 <= 1;
       if(_axi_arvalid && !_axi_arready) begin
         _axi_arvalid <= _axi_arvalid;
       end 
-      if(_axi_rready && _axi_rvalid && (_tmp_26 > 0)) begin
-        _tmp_26 <= _tmp_26 - 1;
+      if(_axi_rready && _axi_rvalid && (counter_27 > 0)) begin
+        counter_27 <= counter_27 - 1;
       end 
-      if((fsm == 2) && ((_axi_arready || !_axi_arvalid) && (_tmp_27 == 0))) begin
+      if((fsm == 2) && ((_axi_arready || !_axi_arvalid) && (counter_28 == 0))) begin
         _axi_araddr <= 2048;
         _axi_arlen <= 127;
         _axi_arvalid <= 1;
-        _tmp_27 <= 128;
+        counter_28 <= 128;
       end 
       __axi_cond_1_1 <= 1;
       if(_axi_arvalid && !_axi_arready) begin
         _axi_arvalid <= _axi_arvalid;
       end 
-      if(_axi_rready && _axi_rvalid && (_tmp_27 > 0)) begin
-        _tmp_27 <= _tmp_27 - 1;
+      if(_axi_rready && _axi_rvalid && (counter_28 > 0)) begin
+        counter_28 <= counter_28 - 1;
       end 
     end
   end
@@ -528,21 +537,21 @@ module main
   assign myaxi_wready = 0;
   reg [32-1:0] fsm;
   localparam fsm_init = 0;
-  reg [9-1:0] _tmp_0;
-  reg [32-1:0] _tmp_1;
-  reg _tmp_2;
-  reg _tmp_3;
-  assign myaxi_arready = (fsm == 0) && !_tmp_2 && _tmp_3;
+  reg [9-1:0] counter_0;
+  reg [32-1:0] addr_1;
+  reg valid_2;
+  reg prev_arvalid_3;
+  assign myaxi_arready = (fsm == 0) && !valid_2 && prev_arvalid_3;
   reg [32-1:0] rdata;
   reg _myaxi_cond_0_1;
 
   always @(posedge CLK) begin
     if(RST) begin
       myaxi_bvalid <= 0;
-      _tmp_3 <= 0;
-      _tmp_1 <= 0;
-      _tmp_0 <= 0;
-      _tmp_2 <= 0;
+      prev_arvalid_3 <= 0;
+      addr_1 <= 0;
+      counter_0 <= 0;
+      valid_2 <= 0;
       myaxi_rdata <= 0;
       myaxi_rvalid <= 0;
       myaxi_rlast <= 0;
@@ -558,19 +567,19 @@ module main
       if(myaxi_wvalid && myaxi_wready && myaxi_wlast) begin
         myaxi_bvalid <= 1;
       end 
-      _tmp_3 <= myaxi_arvalid;
+      prev_arvalid_3 <= myaxi_arvalid;
       if(myaxi_arready && myaxi_arvalid) begin
-        _tmp_1 <= myaxi_araddr;
-        _tmp_0 <= myaxi_arlen + 1;
+        addr_1 <= myaxi_araddr;
+        counter_0 <= myaxi_arlen + 1;
       end 
-      _tmp_2 <= myaxi_arready && myaxi_arvalid;
-      if((fsm == 1) && ((_tmp_0 > 0) && (myaxi_rready || !myaxi_rvalid) && (_tmp_0 > 0))) begin
+      valid_2 <= myaxi_arready && myaxi_arvalid;
+      if((fsm == 1) && ((counter_0 > 0) && (myaxi_rready || !myaxi_rvalid) && (counter_0 > 0))) begin
         myaxi_rdata <= rdata;
         myaxi_rvalid <= 1;
         myaxi_rlast <= 0;
-        _tmp_0 <= _tmp_0 - 1;
+        counter_0 <= counter_0 - 1;
       end 
-      if((fsm == 1) && ((_tmp_0 > 0) && (myaxi_rready || !myaxi_rvalid) && (_tmp_0 > 0)) && (_tmp_0 == 1)) begin
+      if((fsm == 1) && ((counter_0 > 0) && (myaxi_rready || !myaxi_rvalid) && (counter_0 > 0)) && (counter_0 == 1)) begin
         myaxi_rlast <= 1;
       end 
       _myaxi_cond_0_1 <= 1;
@@ -591,15 +600,15 @@ module main
     end else begin
       case(fsm)
         fsm_init: begin
-          if(_tmp_2) begin
-            rdata <= _tmp_1 >> 2;
+          if(valid_2) begin
+            rdata <= addr_1 >> 2;
           end 
-          if(_tmp_2) begin
+          if(valid_2) begin
             fsm <= fsm_1;
           end 
         end
         fsm_1: begin
-          if((_tmp_0 > 0) && (myaxi_rready || !myaxi_rvalid)) begin
+          if((counter_0 > 0) && (myaxi_rready || !myaxi_rvalid)) begin
             rdata <= rdata + 1;
           end 
           if(myaxi_rready && myaxi_rvalid && myaxi_rlast) begin
