@@ -1,7 +1,7 @@
 from __future__ import absolute_import
 from __future__ import print_function
 
-from .axim import AXIM, AXIMLite
+from .axim import AXIM, AXIMLite, AXIMVerify, AXIMLiteVerify
 from .axis import AXIS, AXISLite, AXISRegister, AXISLiteRegister
 from .axistreamin import AXIStreamIn, AXIStreamInFifo
 from .axistreamout import AXIStreamOut, AXIStreamOutFifo
