@@ -98,7 +98,7 @@ class CompileVisitor(ast.NodeVisitor):
     def _variable_type(self, right):
         if isinstance(right, fxd._FixedBase):
             ret = {'type': 'fixed',
-                   'width': max(right.bit_length(), self.datawidth),
+                   'width': max(right.get_width(), self.datawidth),
                    'point': right.point,
                    'signed': right.signed}
             return ret
@@ -508,7 +508,7 @@ class CompileVisitor(ast.NodeVisitor):
         if ln is not None:
             return ln
 
-        return value.bit_length()
+        return vtypes.get_width(value)
 
     def _call_Name_function(self, node, name):
         tree = self.getFunction(name)
